@@ -1,6 +1,7 @@
 #!/usr/bin/python2
 
 import os
+import subprocess
 import sys
 import gtk
 import gobject
@@ -137,7 +138,7 @@ def add_repository_via_cli(line, codename, forceYes, use_ppas):
         with open(file, "w") as text_file:
             text_file.write("%s\n" % deb_line)
             text_file.write("%s\n" % debsrc_line)
-    elif line.startswith("deb "):
+    elif line.startswith("deb ") | line.startswith("http"):
         with open("/etc/apt/sources.list.d/additional-repositories.list", "a") as text_file:
             text_file.write("%s\n" % expand_http_line(line, codename))
 
@@ -1021,7 +1022,7 @@ class Application(object):
         dialog.set_default_response(gtk.RESPONSE_OK)
         response = dialog.run()
         if response == gtk.RESPONSE_OK:
-            os.system("apt-key add %s" % dialog.get_filename())
+            subprocess.call(["apt-key", "add", dialog.get_filename()])
             self.load_keys()
             self.enable_reload_button()
         dialog.destroy()
@@ -1390,9 +1391,8 @@ class Application(object):
 
     def update_apt_cache(self, widget=None):
         self.disable_reload_button()
-        from subprocess import Popen, PIPE
         cmd = ["sudo", "/usr/sbin/synaptic", "--hide-main-window", "--update-at-startup", "--non-interactive"]
-        comnd = Popen(' '.join(cmd), shell=True)
+        comnd = subprocess.Popen(' '.join(cmd), shell=True)
         #returnCode = comnd.wait()
 
     def apply_official_sources(self, widget=None):
@@ -1543,6 +1543,18 @@ if __name__ == "__main__":
 
         (options, args) = parser.parse_args()
 
+        lsb_codename = commands.getoutput("lsb_release -sc")
+        config_dir = "/usr/share/mintsources/%s" % lsb_codename
+        if not os.path.exists(config_dir):
+            print ("LSB codename: '%s'." % lsb_codename)
+            if os.path.exists("/etc/linuxmint/info"):
+                print ("Version of base-files: '%s'." % commands.getoutput("apt version base-files"))
+                print ("Your LSB codename isn't a valid Linux Mint codename.")
+            else:
+                print ("This codename isn't currently supported.")
+            print ("Please check your LSB information with \"lsb_release -a\".")
+            sys.exit(1)
+
         if len(args) > 1 and (args[0] == "add-apt-repository"):
             ppa_line = args[1]
             lsb_codename = commands.getoutput("lsb_release -sc")
